/*
Title:11GHzsrc.c
Description: main file for hc11 code for 11 GHz surplus sources
- quick and dirty for now to get the ball rolling
- I'm not a software engineer if you couldn't tell!

Notes: Language = "Micro-C for the 68hc11" see www.dunfield.com

Created: 5/12/2000 LL WS2B
Last Modified: 5/14/2000 LL -got it working on a couple modules

Copyright(C) 2000 Lance Lascari WS2B
http://www.geocities.com/lance_lascari
lance_lascari@usa.net

*/
             /* for using long library routines */
#define REGISTER

#include "c:\local\personal\micros\uc11\6811reg.h"
#include "c:\local\personal\micros\uc11\6811io.h"
#include "c:\local\personal\micros\uc11\6811int.h"
#include "c:\local\personal\micros\uc11\intvect.h"

/* kludgy way to add code, but without an object linker this is what you get */
#include "c:\local\personal\src\embed_c\sync.c"        /* spi drivers */

#include "ints.c" /* other interrupts */

#define PRESCALER_64 0x8000; /* bit in reference register to set prescaler */
/* 1 = 64, 0=128 */
#define SYNTH_CONTROL_BIT 0x0001; /* 1= R register, 0 = N register */

#define SYNTH_LE_PORT PORTB
#define SYNTH_LE_BIT  0x80
#define SYNTH_SET_LE PORTB|=SYNTH_LE_BIT
#define SYNTH_CLR_LE PORTB&=~SYNTH_LE_BIT
#define INBUFSIZE 10

main()
{

  char input_buffer[INBUFSIZE];
  unsigned fmhz;
  unsigned r;
  unsigned n;
  unsigned p;
  unsigned char a;
  unsigned char ref_div_reg[2];
  unsigned char main_div_reg[3];
  unsigned char flag;

  flag =0;  /* flag is 1 after the synth has been loaded once */  
  /* serial port stuff */

  BAUD=0x30;  /* 9600 with 8 MHz clock */
  SCCR1=0; /* n,8,1, 9th bits not stored, wakeup normal */
  SCCR2=0x0c; /* transmitter and receiver enabled, interrupts off */



 spi_init(SPI_MSTR|SPI_ON|SPI_E_16);  /* CPOL=0,CPHA=0initialize the SPI */

 SYNTH_CLR_LE; /* clear latch enable bit*/
  /* the output frequency of the synthesizer will be 12 x the
frequency that the MB1502 is programmed to ( %2 prescaler in feedback
path, and x6 multiplier chain after vco)....

so, if a user enters a frequency, we need to divide that by 12 before
figuring out what the MB1502 needs to be programmed to.

The initial goal will be to have 1 MHz output steps at 12 GHz, which
means 83.33333 kHz reference on the synthesizer.  Hopefully this is
stable... */

  /* for now, if we keep with this plan, I'll force this to 16 bit
     math which will save quite a bit of space */
  
  p=64;  /* 64 seems like a good number for this prescaler */

  for(;;) /* main loop, repeats all below */
    {
      
      if(!flag)
	{
	  fmhz=12000; /* default freq 12,000 MHz*/
	}
      else
	{
	  printf("enter freq in MHz %d>",fmhz);
	  getstr(input_buffer,INBUFSIZE);
	  fmhz=atoi(input_buffer);
	  if((fmhz<10000) || (fmhz >15000))
	    {
	      printf("\n 10000< Freq < 15000\n, setting to default 12000");
	      fmhz=12000;
	    }
	}

      flag=1; /* now we're past the first pass */

      
      printf("\nfreq in MHz = %d\n",fmhz);

      /* fmhz/=12;*/

      /*  printf("freq in MHz = %d",fmhz);*/
      
      
      /* the following R value will result in 1 MHz steps at 12 GHz with
	 each increment of the overall "N" value in the synth */
      
      r=120;  /* 10 MHz / 120 = 83.3333 kHz */
      
      /* fout = ((p * n) +a) * TCXO_freq/r */
      n=fmhz/p;  /* MB1502 N counter 156 for 10 GHz, 234 for 15 GHz*/
      a=fmhz%p;  /* MB1502 A counter */

      /* debug */      
      printf("\nn=%d a=%d\n",n,a);

      /* now format the bits (easiest for this one since it's only 16 bits)! */
      
      r<<=1;  /* shift to the left one bit to make room for the control bit*/
      r |= SYNTH_CONTROL_BIT; /* set control bit to select R register load */
      r |= PRESCALER_64; /* set prescaler bit to put it in /64 mode */
      
      
      /**** prepare the R register and load it */
      /* r is now set */
      /* lsb --sent second*/
      ref_div_reg[0]=(unsigned char) r&0xff;
      /*msb --sent first*/
      ref_div_reg[1]=(unsigned char) ((r>>8) &0xff);
      
      
      spi_xfer(ref_div_reg[1]);
      spi_xfer(ref_div_reg[0]);
      
      /* this toggling of LE forces a transfer from the shift register
	 into the appropriate register within the PLL chip */
      
      SYNTH_SET_LE; /* set latch enable bit*/
      SYNTH_CLR_LE; /* clear latch enable bit*/
      
      /**** prepare the N register and load it */
      /* here since we've got the pieces already calculated, we'll just go
	 directly to the array that gets loaded */
      
      /* lsb -- gets sent last */
      main_div_reg[0]=a<<1; /* a gets shifted one to make room for control bit*/
      main_div_reg[0] &=~SYNTH_CONTROL_BIT; /* make sure control bit is cleared */
      /* -- middle byte, gets sent second, it's the 8 lowest bits of the n value */
      main_div_reg[1]=(unsigned char)n & 0x00ff;
      /* -- msb, gets sent first, upper 8 bits of the n value) */
      main_div_reg[2]=(unsigned char)((n>>8) & 0xff);
      
      spi_xfer(main_div_reg[2]);
      spi_xfer(main_div_reg[1]);
      spi_xfer(main_div_reg[0]);
      
      
      SYNTH_SET_LE; /* set latch enable bit*/
      SYNTH_CLR_LE; /* clear latch enable bit*/
      
      
      
      
      
      
    }
  

     

  

  
}









